#region Using declarations
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    /// <summary>
    /// Enter the description of your new custom indicator here
    /// </summary>
    [Description("Stocks and Commodities - April 2012 - Sentiment Zone Oscillator")]
    public class SZO : Indicator
    {
        #region Variables
        // Wizard generated variables
            private 	int period 		= 14; // Default setting for Period
			private 	int longPeriod 	= 30;
			private		int percent 	= 95;
        // User defined variables (add any user defined variables below)
			private DataSeries 	R;
			private DataSeries	SP;
			private	DataSeries 	daysUpInPeriod;
			private double 		HLP;
			private	double	 	LLP;
			private	double 		myRange;
			private	double		Prange;
        #endregion

        /// <summary>
        /// This method is used to configure the indicator and is called once before any bar data is loaded.
        /// </summary>
        protected override void Initialize()
        {
            Add(new Plot(Color.FromKnownColor(KnownColor.Black), PlotStyle.Line, "SZOplot"));
			Add(new Plot(Color.FromKnownColor(KnownColor.Red), PlotStyle.Line, "OB"));
			Add(new Plot(Color.FromKnownColor(KnownColor.DarkGreen), PlotStyle.Line, "OS"));
			Add(new Line(Color.FromKnownColor(KnownColor.Gainsboro), 7, "Seven"));
            Add(new Line(Color.FromKnownColor(KnownColor.Gainsboro), -7, "NegSeven"));
			Lines[0].Pen.DashStyle	= DashStyle.Dash;
			Lines[1].Pen.DashStyle	= DashStyle.Dash;
            Overlay				= false;
			R 	= new DataSeries (this);
			SP	= new DataSeries (this);
        }

        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {	
			if (CurrentBar < 1)
			return;
				
			R.Set(Close[0] > Close[1] ? 1: -1);
			SP.Set(TEMA(R, Period)[0]);
			SZOplot.Set(100 * (SP[0]/Period));
			
			//static OB and OS for first LongPeriod bars. 
			if (CurrentBar < LongPeriod)
			{
				OS.Set(7);
				OB.Set(-7);
			}
			
			//calculate and plot dynamic OB/OS levels.
			else if (CurrentBar >= LongPeriod)
			{
				HLP = (MAX(SZOplot, LongPeriod)[0]);
				LLP = (MIN(SZOplot, LongPeriod)[0]);
				myRange = HLP - LLP;
				Prange = myRange * (Percent / 100);
				OB.Set(LLP + Prange);
				OS.Set(HLP + Prange);	
			}
        }

        #region Properties
        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries SZOplot
        {
            get { return Values[0]; }
        }
		
		[Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries OB
        {
            get { return Values[1]; }
        }
		
		[Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries OS
        {
            get { return Values[2]; }
        }

        [Description("Number of bars used in SZO")]
        [GridCategory("Parameters")]
        public int Period
        {
            get { return period; }
            set { period = Math.Max(1, value); }
        }
		
		[Description("Long Period for dynamic OB/OS levels")]
        [GridCategory("Parameters")]
        public int LongPeriod
        {
            get { return longPeriod; }
            set { longPeriod = Math.Max(1, value); }
        }
		
		[Description("Percent input for dynamic OB/OS levels")]
        [GridCategory("Parameters")]
        public int Percent
        {
            get { return percent; }
            set { percent = Math.Max(1, value); }
        }
        #endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private SZO[] cacheSZO = null;

        private static SZO checkSZO = new SZO();

        /// <summary>
        /// Stocks and Commodities - April 2012 - Sentiment Zone Oscillator
        /// </summary>
        /// <returns></returns>
        public SZO SZO(int longPeriod, int percent, int period)
        {
            return SZO(Input, longPeriod, percent, period);
        }

        /// <summary>
        /// Stocks and Commodities - April 2012 - Sentiment Zone Oscillator
        /// </summary>
        /// <returns></returns>
        public SZO SZO(Data.IDataSeries input, int longPeriod, int percent, int period)
        {
            if (cacheSZO != null)
                for (int idx = 0; idx < cacheSZO.Length; idx++)
                    if (cacheSZO[idx].LongPeriod == longPeriod && cacheSZO[idx].Percent == percent && cacheSZO[idx].Period == period && cacheSZO[idx].EqualsInput(input))
                        return cacheSZO[idx];

            lock (checkSZO)
            {
                checkSZO.LongPeriod = longPeriod;
                longPeriod = checkSZO.LongPeriod;
                checkSZO.Percent = percent;
                percent = checkSZO.Percent;
                checkSZO.Period = period;
                period = checkSZO.Period;

                if (cacheSZO != null)
                    for (int idx = 0; idx < cacheSZO.Length; idx++)
                        if (cacheSZO[idx].LongPeriod == longPeriod && cacheSZO[idx].Percent == percent && cacheSZO[idx].Period == period && cacheSZO[idx].EqualsInput(input))
                            return cacheSZO[idx];

                SZO indicator = new SZO();
                indicator.BarsRequired = BarsRequired;
                indicator.CalculateOnBarClose = CalculateOnBarClose;
#if NT7
                indicator.ForceMaximumBarsLookBack256 = ForceMaximumBarsLookBack256;
                indicator.MaximumBarsLookBack = MaximumBarsLookBack;
#endif
                indicator.Input = input;
                indicator.LongPeriod = longPeriod;
                indicator.Percent = percent;
                indicator.Period = period;
                Indicators.Add(indicator);
                indicator.SetUp();

                SZO[] tmp = new SZO[cacheSZO == null ? 1 : cacheSZO.Length + 1];
                if (cacheSZO != null)
                    cacheSZO.CopyTo(tmp, 0);
                tmp[tmp.Length - 1] = indicator;
                cacheSZO = tmp;
                return indicator;
            }
        }
    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Stocks and Commodities - April 2012 - Sentiment Zone Oscillator
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.SZO SZO(int longPeriod, int percent, int period)
        {
            return _indicator.SZO(Input, longPeriod, percent, period);
        }

        /// <summary>
        /// Stocks and Commodities - April 2012 - Sentiment Zone Oscillator
        /// </summary>
        /// <returns></returns>
        public Indicator.SZO SZO(Data.IDataSeries input, int longPeriod, int percent, int period)
        {
            return _indicator.SZO(input, longPeriod, percent, period);
        }
    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Stocks and Commodities - April 2012 - Sentiment Zone Oscillator
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.SZO SZO(int longPeriod, int percent, int period)
        {
            return _indicator.SZO(Input, longPeriod, percent, period);
        }

        /// <summary>
        /// Stocks and Commodities - April 2012 - Sentiment Zone Oscillator
        /// </summary>
        /// <returns></returns>
        public Indicator.SZO SZO(Data.IDataSeries input, int longPeriod, int percent, int period)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.SZO(input, longPeriod, percent, period);
        }
    }
}
#endregion
