#region Using declarations
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Indicator;
using NinjaTrader.Gui.Chart;
using NinjaTrader.Strategy;
#endregion

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    /// <summary>
    /// Stocks and Commodities - April 2012 - Sentiment Zone Oscillator
    /// </summary>
    [Description("Stocks and Commodities - April 2012 - Sentiment Zone Oscillator")]
    public class SZOATS : Strategy
    {
        #region Variables
        // Wizard generated variables
        private int period = 14; // Default setting for Period
        private int longPeriod = 30; // Default setting for LongPeriod
        private int percent = 95; // Default setting for Percent
        // User defined variables (add any user defined variables below)
        #endregion

        /// <summary>
        /// This method is used to configure the strategy and is called once before any strategy method is called.
        /// </summary>
        protected override void Initialize()
        {
            CalculateOnBarClose = true;
			Add(SZO(LongPeriod, Percent, Period));
			EntryHandling = EntryHandling.UniqueEntries;
        }

        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {
			if (CrossAbove(SMA(SZO(LongPeriod, Percent, Period).SZOplot, 30),0, 1)
				&& Close[0] > EMA(60)[0])
			{
				EnterLong("Entry1");
			}
			
			if (Close[0] > EMA(60)[0]
				&& SZO(LongPeriod, Percent, Period).SZOplot[0] < SZO(LongPeriod, Percent, Period).OS[0]
				&& Rising(SMA(SZO(LongPeriod, Percent, Period).SZOplot, 30)))
			{
				EnterLong("Entry2");
			}
			
			if (SMA(SZO(LongPeriod, Percent, Period).SZOplot, 30)[0] > 0
				&& CrossAbove(SZO(LongPeriod, Percent, Period).SZOplot, SZO(LongPeriod, Percent, Period).OS, 1)
				&& Rising(EMA(60)))
			{
				EnterLong("Entry3");
			}
			
			if (CrossAbove(SMA(SZO(LongPeriod, Percent, Period).SZOplot, 30),0, 1))
			{
				ExitLong();	
			}
			
			if (CrossBelow(SZO(LongPeriod, Percent, Period).SZOplot, 7, 1)
				&& Falling (SMA(SZO(LongPeriod, Percent, Period).SZOplot, 30)))
			{
				ExitLong();
			}
        }

        #region Properties
        [Description("Number of bars used in SZO")]
        [GridCategory("Parameters")]
        public int Period
        {
            get { return period; }
            set { period = Math.Max(1, value); }
        }

        [Description("Long Period for dynamic OB/OS levels")]
        [GridCategory("Parameters")]
        public int LongPeriod
        {
            get { return longPeriod; }
            set { longPeriod = Math.Max(1, value); }
        }

        [Description("Percent input for dynamic OB/OS levels")]
        [GridCategory("Parameters")]
        public int Percent
        {
            get { return percent; }
            set { percent = Math.Max(1, value); }
        }
        #endregion
    }
}
